import { memo, useCallback, useEffect, useState } from "react";
import { setStorage } from "@/utils/navigation";
import { ChevronDownIcon, ChevronUpIcon } from "@chakra-ui/icons";
import {
  Avatar,
  Box,
  Button,
  Fade,
  Flex,
  Icon,
  Text,
  useColorModeValue,
} from "@chakra-ui/react";
import { useRouter } from "next/navigation";
import moment from "moment";
import { BadgeColorKey } from "@/utils/constant";
import { CustomBadge } from "@/app/admin/settings/assignment-priority/component/table";
import { useSession } from "next-auth/react";
import { ProjectInterface } from "@/types/project";
import { Assignment } from "@/types/assignment";

export const Card = ({ consumer }: { consumer: Assignment }) => {
  const [isOpen, setIsOpen] = useState(false);
  const { data: session } = useSession();
  const [project, setProject] = useState<ProjectInterface>(null);
  const router = useRouter();
  const textColor = useColorModeValue("red.700", "red.300");

  const toggleCard = () => {
    setIsOpen((prev) => !prev);
  };

  const handleCreateConsumer = useCallback(async () => {
    const data = {
      assignment_id: consumer?.id ?? null,
      case_id: consumer?.case_id ?? null,
      batch_id: consumer?.batch_id ?? null,
      consumer_id: consumer?.consumer_id ?? null,
      case: consumer?.case ?? null,
    };
    setStorage("attempted", data);
    setStorage("attempted_update", data);
    router.push("/officer/consumers/store");
  }, [consumer, router]);

  const handleCreateKyc = useCallback(async () => {
    const data = {
      assignment_id: consumer?.id ?? null,
      case_id: consumer?.case_id ?? null,
      batch_id: consumer?.batch_id ?? null,
      kyc_id: consumer?.kyc_id ?? null,
      case: consumer?.case ?? null,
    };
    setStorage("kyc", data);
    setStorage("kyc_update", data);
    router.push("/officer/kycs/store");
  }, [consumer, router]);

  const handleCreateDC = useCallback(async () => {
    const data = {
      assignment_id: consumer?.id ?? null,
      case_id: consumer?.case_id ?? null,
      batch_id: consumer?.batch_id ?? null,
      dc_form_id: consumer?.dc_form_id ?? null,
      case: consumer?.case ?? null,
    };
    setStorage("dc", data);
    setStorage("dc_update", data);
    router.push("/officer/dc/store");
  }, [consumer, router]);

  useEffect(() => {
    if (session?.projects) {
      setProject(session?.projects[0]);
    }
  }, [session?.projects]);

  return (
    <Box border="1px" p={3} rounded={20} mt="30px">
      <Flex
        gap={2}
        alignItems="center"
        w="100%"
        cursor="pointer"
        onClick={toggleCard}
      >
        <Box>
          <Avatar name={consumer?.case?.name} size="sm" />
        </Box>
        <Text
          fontSize="lg"
          fontWeight="bold"
          sx={{ textTransform: "uppercase" }}
        >
          {consumer?.case?.name} &nbsp; {consumer?.case?.contract} |{" "}
          {consumer?.case?.contract_account} | CD:{" "}
          {consumer?.case?.cycle_day?.cycle_day} | MRU: {consumer?.case?.mru}
        </Text>

        <CustomBadge
          title={consumer?.assignment_priority?.name}
          color={consumer?.assignment_priority?.color as BadgeColorKey}
        />

        <Box ms="auto">
          <Icon
            as={isOpen ? ChevronUpIcon : ChevronDownIcon}
            width="20px"
            height="20px"
            color="inherit"
          />
        </Box>
      </Flex>

      {isOpen && (
        <Fade in={isOpen}>
          <Box py="1" mt={3} px={0} rounded="md">
            <Flex
              rowGap={1}
              alignItems="center"
              flexWrap="wrap"
              justifyContent="space-between"
            >
              <Box width="50%">
                <Text
                  fontSize="sm"
                  fontWeight="bold"
                  sx={{ textTransform: "capitalized" }}
                >
                  Consumer No: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {consumer?.case?.consumer_no}
                  </Text>
                </Text>
              </Box>
              <Box width="50%">
                <Text
                  fontSize="sm"
                  fontWeight="bold"
                  sx={{ textTransform: "capitalized" }}
                >
                  Address: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {consumer?.case?.address}
                  </Text>
                </Text>
              </Box>
              <Box width={"50%"}>
                <Text
                  fontSize="sm"
                  fontWeight="bold"
                  sx={{ textTransform: "capitalized" }}
                >
                  Meter No: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {consumer?.case?.meter_no}
                  </Text>
                </Text>
              </Box>
              <Box width={"50%"}>
                <Text
                  fontSize="sm"
                  fontWeight="bold"
                  sx={{ textTransform: "capitalized" }}
                >
                  Total Due Units: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {consumer?.case.total_due_units}
                  </Text>
                </Text>
              </Box>
              <Box width={"50%"}>
                <Text
                  fontSize="sm"
                  fontWeight="bold"
                  sx={{ textTransform: "capitalized" }}
                >
                  Total Due Billing: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {consumer?.case?.total_due_billing}
                  </Text>
                </Text>
              </Box>
              <Box width={"50%"}>
                <Text
                  fontSize="sm"
                  fontWeight="bold"
                  sx={{ textTransform: "capitalized" }}
                >
                  Last Payment Amount: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {consumer?.case?.last_payment_amount}
                  </Text>
                </Text>
              </Box>
              <Box width={"50%"}>
                <Text
                  fontSize="sm"
                  fontWeight="bold"
                  sx={{ textTransform: "capitalized" }}
                >
                  Last Payment Date: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {consumer?.case?.last_payment_date}
                  </Text>
                </Text>
              </Box>
              <Box width={"50%"}>
                <Text
                  fontSize="sm"
                  fontWeight="bold"
                  sx={{ textTransform: "capitalized" }}
                >
                  MRU: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {consumer?.case?.mru}
                  </Text>
                </Text>
              </Box>
              <Box width={"50%"}>
                <Text
                  fontSize="sm"
                  fontWeight="bold"
                  sx={{ textTransform: "capitalized" }}
                >
                  Rate Tariff: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {consumer?.case?.rate_tariff}
                  </Text>
                </Text>
              </Box>
              <Box width={"50%"}>
                <Text
                  fontSize="sm"
                  fontWeight="bold"
                  sx={{ textTransform: "capitalized" }}
                >
                  Dues: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {consumer?.case?.dues}
                  </Text>
                </Text>
              </Box>
              <Box width={"50%"}>
                <Text
                  fontSize="sm"
                  fontWeight="bold"
                  sx={{ textTransform: "capitalized" }}
                >
                  Ibc: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {consumer?.case?.ibc?.area_name}
                  </Text>
                </Text>
              </Box>
              <Box width={"50%"}>
                <Text
                  fontSize="sm"
                  fontWeight="bold"
                  sx={{ textTransform: "capitalized" }}
                >
                  Cycle Days: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {consumer?.case?.cycle_day?.cycle_day}
                  </Text>
                </Text>
              </Box>
              <Box width="50%">
                <Text fontSize="sm" fontWeight="bold">
                  Created At: &nbsp;
                  <Text as="span" fontWeight="normal">
                    {moment(consumer?.created_at).fromNow()}
                  </Text>
                </Text>
              </Box>
            </Flex>
          </Box>
        </Fade>
      )}

      <Flex columnGap={1} justify="space-between" alignItems="center" mt="6">
        <Flex gap={2} alignItems="center">
          {(consumer?.attempt == null ||
            consumer?.attempt?.consumer_id == null) &&
            project?.consumer_form === 1 && (
              <Button onClick={handleCreateConsumer} size="sm" px="3">
                Attempt RO
              </Button>
            )}
          {session?.user?.permissions?.includes("kyc create") &&
            (consumer?.attempt == null || consumer?.attempt?.kyc_id == null) &&
            project?.kyc_form === 1 && (
              <Button onClick={handleCreateKyc} size="sm" px="3">
                Attempt KYC
              </Button>
            )}

          {session?.user?.permissions?.includes("dc-form create") &&
            (consumer?.attempt == null ||
              consumer?.attempt?.dc_form_id == null) &&
            project?.dc_form === 1 && (
              <Button onClick={handleCreateDC} size="sm" px="3">
                Attempt DC
              </Button>
            )}
        </Flex>

        <Text
          fontSize="md"
          fontWeight="bold"
          color={textColor}
          sx={{ textTransform: "uppercase" }}
        >
          Deadline: {moment(consumer?.due_date).endOf("day").fromNow()}
        </Text>
      </Flex>
    </Box>
  );
};

export default memo(Card);
